@@ -14,10 +14,9 @@ from os import path
 from docutils import nodes
 from docutils.io import StringInput
 from docutils.utils import relative_path
-from docutils.transforms import Transform
 
 from sphinx import addnodes
-from sphinx.util import split_index_msg
+from sphinx.util import split_index_msg, logging
 from sphinx.util.i18n import find_catalog
 from sphinx.util.nodes import (
     LITERAL_TYPE_NODES, IMAGE_TYPE_NODES,
@@ -25,10 +24,20 @@ from sphinx.util.nodes import (
 )
 from sphinx.util.pycompat import indent
 from sphinx.locale import init as init_locale
+from sphinx.transforms import SphinxTransform
 from sphinx.domains.std import make_glossary_term, split_term_classifiers
 
+if False:
+    # For type annotation
+    from typing import Any, Dict, List, Tuple  # NOQA
+    from sphinx.application import Sphinx  # NOQA
+    from sphinx.config import Config  # NOQA
+
+logger = logging.getLogger(__name__)
+
 
 def publish_msgstr(app, source, source_path, source_line, config, settings):
+    # type: (Sphinx, unicode, unicode, int, Config, Dict) -> nodes.document
     """Publish msgstr (single line) into docutils document
 
     :param sphinx.application.Sphinx app: sphinx application
@@ -43,7 +52,7 @@ def publish_msgstr(app, source, source_path, source_line, config, settings):
     from sphinx.io import SphinxI18nReader
     reader = SphinxI18nReader(
         app=app,
-        parsers=config.source_parsers,
+        parsers=app.registry.get_source_parsers(),
         parser_name='restructuredtext',  # default parser
     )
     reader.set_lineno_for_reporter(source_line)
@@ -59,37 +68,38 @@ def publish_msgstr(app, source, source_path, source_line, config, settings):
     return doc
 
 
-class PreserveTranslatableMessages(Transform):
+class PreserveTranslatableMessages(SphinxTransform):
     """
     Preserve original translatable messages befor translation
     """
     default_priority = 10  # this MUST be invoked before Locale transform
 
     def apply(self):
+        # type: () -> None
         for node in self.document.traverse(addnodes.translatable):
             node.preserve_original_messages()
 
 
-class Locale(Transform):
+class Locale(SphinxTransform):
     """
     Replace translatable nodes with their translated doctree.
     """
     default_priority = 20
 
     def apply(self):
-        env = self.document.settings.env
+        # type: () -> None
         settings, source = self.document.settings, self.document['source']
         # XXX check if this is reliable
-        assert source.startswith(env.srcdir)
-        docname = path.splitext(relative_path(path.join(env.srcdir, 'dummy'),
+        assert source.startswith(self.env.srcdir)
+        docname = path.splitext(relative_path(path.join(self.env.srcdir, 'dummy'),
                                               source))[0]
         textdomain = find_catalog(docname,
                                   self.document.settings.gettext_compact)
 
         # fetch translations
-        dirs = [path.join(env.srcdir, directory)
-                for directory in env.config.locale_dirs]
-        catalog, has_catalog = init_locale(dirs, env.config.language, textdomain)
+        dirs = [path.join(self.env.srcdir, directory)
+                for directory in self.config.locale_dirs]
+        catalog, has_catalog = init_locale(dirs, self.config.language, textdomain)
         if not has_catalog:
             return
 
@@ -114,8 +124,8 @@ class Locale(Transform):
             if isinstance(node, LITERAL_TYPE_NODES):
                 msgstr = '::\n\n' + indent(msgstr, ' ' * 3)
 
-            patch = publish_msgstr(
-                env.app, msgstr, source, node.line, env.config, settings)
+            patch = publish_msgstr(self.app, msgstr, source,
+                                   node.line, self.config, settings)
             # XXX doctest and other block markup
             if not isinstance(patch, nodes.paragraph):
                 continue  # skip for now
@@ -176,6 +186,7 @@ class Locale(Transform):
 
                     # replace target's refname to new target name
                     def is_named_target(node):
+                        # type: (nodes.Node) -> bool
                         return isinstance(node, nodes.target) and  \
                             node.get('refname') == old_name
                     for old_target in self.document.traverse(is_named_target):
@@ -185,16 +196,16 @@ class Locale(Transform):
 
             # glossary terms update refid
             if isinstance(node, nodes.term):
-                gloss_entries = env.temp_data.setdefault('gloss_entries', set())
+                gloss_entries = self.env.temp_data.setdefault('gloss_entries', set())
                 for _id in node['names']:
                     if _id in gloss_entries:
                         gloss_entries.remove(_id)
 
                     parts = split_term_classifiers(msgstr)
-                    patch = publish_msgstr(
-                        env.app, parts[0], source, node.line, env.config, settings)
-                    patch = make_glossary_term(
-                        env, patch, parts[1], source, node.line, _id)
+                    patch = publish_msgstr(self.app, parts[0], source,
+                                           node.line, self.config, settings)
+                    patch = make_glossary_term(self.env, patch, parts[1],
+                                               source, node.line, _id)
                     node['ids'] = patch['ids']
                     node['names'] = patch['names']
                     processed = True
@@ -204,12 +215,12 @@ class Locale(Transform):
                 for child in patch.children:
                     child.parent = node
                 node.children = patch.children
-                node['translated'] = True
+                node['translated'] = True  # to avoid double translation
 
         # phase2: translation
         for node, msg in extract_messages(self.document):
-            if node.get('translated', False):
-                continue
+            if node.get('translated', False):  # to avoid double translation
+                continue  # skip if the node is already translated by phase1
 
             msgstr = catalog.gettext(msg)
             # XXX add marker to untranslated parts
@@ -239,8 +250,8 @@ class Locale(Transform):
             if isinstance(node, LITERAL_TYPE_NODES):
                 msgstr = '::\n\n' + indent(msgstr, ' ' * 3)
 
-            patch = publish_msgstr(
-                env.app, msgstr, source, node.line, env.config, settings)
+            patch = publish_msgstr(self.app, msgstr, source,
+                                   node.line, self.config, settings)
             # XXX doctest and other block markup
             if not isinstance(
                     patch,
@@ -249,10 +260,12 @@ class Locale(Transform):
 
             # auto-numbered foot note reference should use original 'ids'.
             def is_autonumber_footnote_ref(node):
+                # type: (nodes.Node) -> bool
                 return isinstance(node, nodes.footnote_reference) and \
                     node.get('auto') == 1
 
             def list_replace_or_append(lst, old, new):
+                # type: (List, Any, Any) -> None
                 if old in lst:
                     lst[lst.index(old)] = new
                 else:
@@ -260,9 +273,9 @@ class Locale(Transform):
             old_foot_refs = node.traverse(is_autonumber_footnote_ref)
             new_foot_refs = patch.traverse(is_autonumber_footnote_ref)
             if len(old_foot_refs) != len(new_foot_refs):
-                env.warn_node('inconsistent footnote references in '
-                              'translated message', node)
-            old_foot_namerefs = {}
+                logger.warning('inconsistent footnote references in translated message',
+                               location=node)
+            old_foot_namerefs = {}  # type: Dict[unicode, List[nodes.footnote_reference]]
             for r in old_foot_refs:
                 old_foot_namerefs.setdefault(r.get('refname'), []).append(r)
             for new in new_foot_refs:
@@ -290,13 +303,14 @@ class Locale(Transform):
             # * use translated refname for section refname.
             # * inline reference "`Python <...>`_" has no 'refname'.
             def is_refnamed_ref(node):
+                # type: (nodes.Node) -> bool
                 return isinstance(node, nodes.reference) and  \
                     'refname' in node
             old_refs = node.traverse(is_refnamed_ref)
             new_refs = patch.traverse(is_refnamed_ref)
             if len(old_refs) != len(new_refs):
-                env.warn_node('inconsistent references in '
-                              'translated message', node)
+                logger.warning('inconsistent references in translated message',
+                               location=node)
             old_ref_names = [r['refname'] for r in old_refs]
             new_ref_names = [r['refname'] for r in new_refs]
             orphans = list(set(old_ref_names) - set(new_ref_names))
@@ -315,6 +329,7 @@ class Locale(Transform):
 
             # refnamed footnote and citation should use original 'ids'.
             def is_refnamed_footnote_ref(node):
+                # type: (nodes.Node) -> bool
                 footnote_ref_classes = (nodes.footnote_reference,
                                         nodes.citation_reference)
                 return isinstance(node, footnote_ref_classes) and \
@@ -323,8 +338,8 @@ class Locale(Transform):
             new_refs = patch.traverse(is_refnamed_footnote_ref)
             refname_ids_map = {}
             if len(old_refs) != len(new_refs):
-                env.warn_node('inconsistent references in '
-                              'translated message', node)
+                logger.warning('inconsistent references in translated message',
+                               location=node)
             for old in old_refs:
                 refname_ids_map[old["refname"]] = old["ids"]
             for new in new_refs:
@@ -339,10 +354,11 @@ class Locale(Transform):
             new_refs = patch.traverse(addnodes.pending_xref)
             xref_reftarget_map = {}
             if len(old_refs) != len(new_refs):
-                env.warn_node('inconsistent term references in '
-                              'translated message', node)
+                logger.warning('inconsistent term references in translated message',
+                               location=node)
 
             def get_ref_key(node):
+                # type: (nodes.Node) -> Tuple[unicode, unicode, unicode]
                 case = node["refdomain"], node["reftype"]
                 if case == ('std', 'term'):
                     return None
@@ -379,12 +395,12 @@ class Locale(Transform):
             if isinstance(node, IMAGE_TYPE_NODES):
                 node.update_all_atts(patch)
 
-            node['translated'] = True
+            node['translated'] = True  # to avoid double translation
 
-        if 'index' in env.config.gettext_additional_targets:
+        if 'index' in self.config.gettext_additional_targets:
             # Extract and translate messages for index entries.
             for node, entries in traverse_translatable_index(self.document):
-                new_entries = []
+                new_entries = []   # type: List[Tuple[unicode, unicode, unicode, unicode, unicode]]  # NOQA
                 for type, msg, tid, main, key_ in entries:
                     msg_parts = split_index_msg(type, msg)
                     msgstr_parts = []
@@ -399,18 +415,23 @@ class Locale(Transform):
                 node['raw_entries'] = entries
                 node['entries'] = new_entries
 
+        # remove translated attribute that is used for avoiding double translation.
+        def has_translatable(node):
+            return isinstance(node, nodes.Element) and 'translated' in node
+        for node in self.document.traverse(has_translatable):
+            node.delattr('translated')
 
-class RemoveTranslatableInline(Transform):
+
+class RemoveTranslatableInline(SphinxTransform):
     """
     Remove inline nodes used for translation as placeholders.
     """
     default_priority = 999
 
     def apply(self):
+        # type: () -> None
         from sphinx.builders.gettext import MessageCatalogBuilder
-        env = self.document.settings.env
-        builder = env.app.builder
-        if isinstance(builder, MessageCatalogBuilder):
+        if isinstance(self.app.builder, MessageCatalogBuilder):
             return
         for inline in self.document.traverse(nodes.inline):
             if 'translatable' in inline:
